﻿using System;
using Pixiview.Utils;
using Xamarin.Forms;

namespace Pixiview
{
    public partial class AppShell : Shell
    {
        public static new AppShell Current => Shell.Current as AppShell;

        public static Thickness NavigationBarOffset { get; private set; }
        public static Thickness HalfNavigationBarOffset { get; private set; }
        public static Thickness TotalBarOffset { get; private set; }

        public event EventHandler<BarHeightEventArgs> NavigationBarHeightChanged;
        public event EventHandler<BarHeightEventArgs> StatusBarHeightChanged;

        public AppShell()
        {
            InitializeComponent();

            App.DebugPrint($"folder: {Stores.PersonalFolder}");
        }

        public void SetNavigationBarHeight(double height)
        {
            NavigationBarOffset = new Thickness(0, height, 0, 0);
            HalfNavigationBarOffset = new Thickness(0, height / 2, 0, 0);

            NavigationBarHeightChanged?.Invoke(this, new BarHeightEventArgs
            {
                NavigationBarHeight = height
            });
        }

        public void SetStatusBarHeight(double height)
        {
            TotalBarOffset = new Thickness(0, NavigationBarOffset.Top + height, 0, 0);

            StatusBarHeightChanged?.Invoke(this, new BarHeightEventArgs
            {
                StatusBarHeight = height
            });
        }
    }

    public class BarHeightEventArgs : EventArgs
    {
        public double NavigationBarHeight { get; set; }
        public double StatusBarHeight { get; set; }
    }
}
